<template>
    <div class="design-tree-container">
        <context-menu-container>
            <el-tree
            v-if="inited"
            ref="${ctrl.name}"
            class="design-tree"
            node-key="id"
            lazy
            :default-expanded-keys="expandedKeys"
            :props="{
                label: 'text',
                isLeaf: 'leaf'
            }"
            :load="load"
            :highlight-current="true"
            :expand-on-click-node="false"
            @current-change="selectionChange"
            :filter-node-method="filterNode"
            >
                <div slot-scope="{ node, data }">
                <!-- <context-menu contextMenuStyle={{ width: '100%' }} data={node} renderContent={this.renderContextMenu}> -->
                    <tooltip transfer style="width: 100%;" max-width="2000" placement="right">
                        <div class="tree-node" @dblclick="doDefaultAction(node)">
                            <span class="icon">
                                <i v-if=" data.iconcls && !Object.is(data.iconcls, '')" :class="data.iconcls"></i>
                                <img v-else-if="data.icon && !Object.is(data.icon, '')" :src="data.icon" />
                                <icon v-else-if="isOutputIconDefault" type="ios-paper-outline"></icon>&nbsp;
                            </span>
                            <span class="text">
                                <span v-if="data.html" v-html="data.html"></span>
                                <span v-else>{{data.text}}</span>
                            </span>
                        </div>
                        <template slot="content">
                            <span v-if="data.html" v-html="data.html"></span>
                            <span v-else>{{ data.text }}</span>
                        </template>
                    </tooltip>
                <!-- </context-menu> -->
                </div>
            </el-tree>
        </context-menu-container>
    </div>
</template>
<#ibizinclude>
../@MACRO/CONTROL/CONTROL_HEADER-BASE.vue.ftl
</#ibizinclude>

    /**
     * 获取多项数据
     *
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getDatas(): any[] {
        return [this.selectData];
    }

    /**
     * 获取单项树
     *
     * @returns {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    public getData(): any {
        return this.selectData;
    }

    /**
     * 是否默认选中第一条数据
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop({ default: false }) protected isSelectFirstDefault!: boolean;

    /**
     * 显示处理提示
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop({ default: true }) protected showBusyIndicator?: boolean;

    /**
     * 初始化完成
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected inited: boolean = false;

    /**
     * 选中数据
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected selectData: any = {};

    /**
     * 部件行为--update
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected updateAction!: string;

    /**
     * 部件行为--fetch
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected fetchAction!: string;

    /**
     * 部件行为--remove
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected removeAction!: string;

    /**
     * 部件行为--load
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected loadAction!: string;

    /**
     * 部件行为--create
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Prop() protected createAction!: string;

    /**
     * 父数据
     *
     * @type {*}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected parentData: any = {};

    /**
     * 过滤属性
     *
     * @type {string}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected srfnodefilter: string = '';

    /**
     * 默认输出图标
     *
     * @type {boolean}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected isOutputIconDefault: boolean = ${ctrl.isOutputIconDefault()?c};

    /**
     * 树数据
     *
     * @type {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Provide()
    protected nodes: any[] = [];

    /**
     * 数据展开主键
     *
     * @type {string[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    @Provide()
    protected expandedKeys: string[] = [];

    /**
     * 选中数据变更事件
     *
     * @private
     * @param {*} data
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    private selectionChange(data: any, node: any) {
        this.selectData = {};
        if (data.disableselect) {
            node.isCurrent = false;
            this.$emit('selectionchange', []);
            return;
        }
        Object.assign(this.selectData, data);
        this.$emit('selectionchange', [this.selectData]);
    }

    /**
     * Vue声明周期(处理组件的输入属性)
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected created() {
        this.afterCreated();
    }

    /**
     * 执行created后的逻辑
     *
     *  @memberof ${srfclassname('${ctrl.codeName}')}
     */    
    protected afterCreated(){
        if (this.viewState) {
            this.viewStateEvent = this.viewState.subscribe(({ tag, action, data }) => {
                if (!Object.is(tag, this.name)) {
                    return;
                }
                if (Object.is('load', action)) {
                    if (data) {
                        Object.assign(this.parentData, data);
                    }
                    this.inited = false;
                    this.$nextTick(() => {
                        this.inited = true;
                    });
                }
                if (Object.is('filter', action)) {
                    this.srfnodefilter  = data.srfnodefilter;
                    (this.$refs.${ctrl.name} as any).filter(this.srfnodefilter);
                }
                if (Object.is('refresh_parent', action)) {
                    this.refresh_parent();
                }
            });
        }
    }

    /**
     * 对树节点进行筛选操作
     * @memberof OrderTree
     */
    protected filterNode(value:any, data:any) {
        if (!value) return true;
        return data.text.indexOf(value) !== -1;
    }


    /**
     * vue 生命周期
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected destroyed() {
        this.afterDestroy();
    }

    /**
     * 执行destroyed后的逻辑
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected afterDestroy() {
        if (this.viewStateEvent) {
            this.viewStateEvent.unsubscribe();
        }
        <#if destroyed_block??>
        ${destroyed_block}
        </#if>
    }

    /**
     * 刷新数据
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected refresh_all(): void {
        this.inited = false;
        this.$nextTick(() => {
            this.inited = true;
        });
    }

    /**
     * 刷新父节点
     *
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected refresh_parent(): void {
        if (Object.keys(this.selectData).length === 0) {
            return;
        }
        const tree: any = this.$refs.${ctrl.name};
        const node: any = tree.getNode(this.selectData.id);
        if (!node || !node.parent) {
            return;
        }
        const { parent: _parent } = node;
        const id: string = _parent.key ? _parent.key : '#';
        const param: any = { srfnodeid: id };
        Object.assign(param, this.parentData);
        this.refresh_node(param, true);
    }

    /**
     * 数据加载
     *
     * @param {*} node
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected load(node: any = {}, resolve?: any) {
        if (node.data && node.data.children) {
            resolve(node.data.children);
            return;
        }
        const params: any = {
            srfnodeid: node.data && node.data.id ? node.data.id : "#",
            srfnodefilter: this.srfnodefilter
        };
        Object.assign(params, this.parentData);
        Object.assign(params,{viewparams:this.viewparams});
        this.service.getNodes(JSON.parse(JSON.stringify(this.context)),params).then((response: any) => {
            if (!response || response.status !== 200) {
                this.$Notice.error({ title: "错误", desc: response.info });
                resolve([]);
                return;
            }
            const _items = response.data;
            this.formatExpanded(_items);
            if (!node.level || node.level === 0) {
                this.nodes = [..._items];
            }
            this.nodes = response.data;
            resolve([..._items]);
            if(this.isSelectFirstDefault){
                this.setDefaultSelection(_items, true);
            }
            this.$emit("load", _items);
        }).catch((response: any) => {
            resolve([]);
            if (response && response.status === 401) {
                return;
            }
            this.$Notice.error({ title: "错误", desc: response.info });
        });
    }

    /**
     * 刷新功能
     *
     * @param {any[]} args
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected refresh(args: any[]): void {
        if (Object.keys(this.selectData).length === 0) {
            return;
        }
        const id: string = this.selectData.id;
        const param: any = { srfnodeid: id };
        Object.assign(param, this.parentData);
        this.refresh_node(param, false);
    }

    /**
     * 刷新节点
     *
     * @private
     * @param {*} [arg={}]
     * @param {boolean} parentnode
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    private refresh_node(arg: any = {}, parentnode: boolean): void {
        const { srfnodeid: id } = arg;
        Object.assign(arg,{viewparams:this.viewparams});
        const get: Promise<any> = this.service.getNodes(JSON.parse(JSON.stringify(this.context)),arg);
        get.then((response: any) => {
            if (!response || response.status !== 200) {
                this.$Notice.error({ title: '错误', desc: response.info });
                return;
            }
            const _items = [...response.data];
            this.formatExpanded(_items);
            const tree: any = this.$refs.${ctrl.name};
            tree.updateKeyChildren(id, _items);
            if (parentnode) {
                this.selectData = {};
            }
            if(this.isSelectFirstDefault){
                this.setDefaultSelection(_items);
            }
        }).catch((response: any) => {
            if (response && response.status === 401) {
                return;
            }
            this.$Notice.error({ title: '错误', desc: response.info });
        });
    }

    /**
     * 格式化数据
     *
     * @private
     * @param {any[]} items
     * @returns {any[]}
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    private formatExpanded(items: any[]): any[] {
        const data: any[] = [];
        items.forEach((item) => {
            if (item.expanded || (item.children && item.children.length > 0)) {
                this.expandedKeys.push(item.id);
            }
        });
        return data;
    }

    /**
     * 设置默认选中
     *
     * @param {any[]} items
     * @param {any[]} isRoot 是否是加载根节点
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected setDefaultSelection(items: any[], isRoot:boolean = false): void {
        let index = items.findIndex((item: any) => item.selected);
        if(index === -1) {
            if(isRoot){
                index = 0;
            }else{
                return;
            }
        }
        const data = items[index];
        this.setTreeNodeHighLight(data);
        this.$emit('selectionchange', [this.selectData]);
    } 

    /**
     * 设置选中项
     *
     * @param {*} data
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected setTreeNodeHighLight(data: any): void {
        this.selectData = {};
        Object.assign(this.selectData, data);
        const tree: any = this.$refs.${ctrl.name};
        tree.setCurrentKey(data.id);
    }

    /**
     * 绘制右击菜单
     *
     * @param {*} node
     * @returns
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    protected renderContextMenu(node: any) {
        let content;
        if (node && node.data) {
            const data: any = node.data;
            this.selectData = { ...data };
            const tags: string[] = data.id.split(';');
<#--  <#if ctrl.getPSControls()??>
<#list ctrl.getPSControls() as childCtrl>
<#if childCtrl.getControlType() == "CONTEXTMENU">
            if (tags[0] === "${childCtrl.getOwner().getNodeType()}") {
                content = this.renderContextMenu${srfclassname(childCtrl.getOwner().getNodeType()?lower_case)}();
            }
</#if>
</#list>
</#if>  -->
        }
        return content;
    }
<#--  <#if ctrl.getPSControls()??>
<#list ctrl.getPSControls() as childCtrl>
<#if childCtrl.getControlType() == "CONTEXTMENU">

    /**
     * 绘制${childCtrl.getOwner().getNodeType()}类型右键菜单
     *
     * @param {*} node
     * @returns
     * @memberof ${srfclassname('${ctrl.codeName}')}
     */
    private renderContextMenu${srfclassname(childCtrl.getOwner().getNodeType()?lower_case)}() {
        return (
<@ibizindent blank=12>
${P.getCtrlCode(childCtrl, 'CONTROL.html').code}
</@ibizindent>
        );
    }
</#if>
</#list>
</#if>  -->

    /**
     * 执行默认界面行为
     *
     * @param {*} node
     * @memberof AppView
     */
    protected doDefaultAction(node: any) {
        if (node && node.data) {
            const data: any = node.data;
            const tags: string[] = data.id.split(';');
<#if ctrl.getPSDETreeNodes()??>
<#list ctrl.getPSDETreeNodes() as node>
<#if node.getPSDEContextMenu()?? && node.getDefaultPSUIAction()??>
<#assign contextMenu = node.getPSDEContextMenu()/>
            if (tags[0] === "${contextMenu.getOwner().getNodeType()}") {
                this.${contextMenu.name}_click({tag: '${node.getDefaultPSUIAction().name}'});
            }
</#if>
</#list>
</#if>
        }
    }

<#ibizinclude>
../@MACRO/CONTROL/CONTROL_BOTTOM-BASE.vue.ftl
</#ibizinclude>

<#ibizinclude>
../@MACRO/CONTROL/CONTROL-BASE.style.ftl
</#ibizinclude>